package cn.sh.stc.sict.theme.hphy.controller.mp;


import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.sh.stc.sict.cloud.common.core.util.R;
import cn.sh.stc.sict.theme.hpgp.model.HpgpDepartmentRank;
import cn.sh.stc.sict.theme.hpgp.service.HpgpBusyIdlePredictionService;
import cn.sh.stc.sict.theme.hpgp.service.HpgpDepartmentRankService;
import cn.sh.stc.sict.theme.hphy.model.HpDeptInfo;
import cn.sh.stc.sict.theme.hphy.service.HpDeptInfoService;
import com.baomidou.mybatisplus.extension.api.ApiController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

/**
 * (HpDeptInfo)维护接口
 *
 * @author makejava
 * @since 2021-01-28 10:35:58
 */
@Api(value = "[C] 黄浦预约——科室信息接口", tags = "[C] 黄浦预约——科室信息接口")
@RestController
@RequestMapping("/hphy/c/dept")
@AllArgsConstructor
public class HpDeptInfoController extends ApiController {
    /**
     * 服务对象
     */
    private final HpDeptInfoService hpDeptInfoService;
    private final HpgpDepartmentRankService hpgpDepartmentRankService;
    private final HpgpBusyIdlePredictionService hpgpBusyIdlePredictionService;

    @ApiOperation("科室医生查询接口")
    @GetMapping
    public R<?> getDeptDoctors(@ApiParam("医院代码") @RequestParam("hospitalCode") String hospitalCode,
                               @ApiParam("科室代码") @RequestParam("deptCode") String deptCode) {
        return new R<>(hpDeptInfoService.getDeptDoctors(hospitalCode, deptCode));
    }


    @ApiOperation("更新科室号源信息")
    @PostMapping("/wd/resource")
    public R updateDeptResource() {
        List<HpDeptInfo> deptList = hpDeptInfoService.list();
        deptList.forEach(dept -> {
            if (StrUtil.isEmpty(dept.getOneDeptCode()) || StrUtil.isEmpty(dept.getDeptCode())) {
                return;
            }
            hpDeptInfoService.asyncUpdate(dept);
        });
        return new R();
    }

    @ApiOperation("更新科室排行表号源信息")
    @PostMapping("/rank/wd/resource")
    public R updateRankDeptResource() {
        hpgpDepartmentRankService.updateResource();
        return new R();
    }

    @ApiOperation("定时统计排行科室的号源信息")
    @PostMapping("/statistic/rank/wd/resource")
    public R statisticRankDeptResourceInfo() {
        List<HpgpDepartmentRank> deptList = hpgpDepartmentRankService.list();
        deptList.forEach(dept -> {
            if (StrUtil.isEmpty(dept.getOneDeptCode()) || StrUtil.isEmpty(dept.getDeptCode())) {
                return;
            }
            Date startDate = DateUtil.date();
            Date endDate = DateUtil.offsetDay(startDate, 10);
            hpgpBusyIdlePredictionService.statisticResourceInfo(dept, startDate, endDate);
        });
        return new R();
    }
}
