package cn.sh.stc.sict.theme.hphy.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.json.JSONUtil;
import cn.sh.stc.sict.theme.hpgp.model.HpgpDepartmentRank;
import cn.sh.stc.sict.theme.hphy.constant.DataConstant;
import cn.sh.stc.sict.theme.hphy.dao.HpDeptInfoMapper;
import cn.sh.stc.sict.theme.hphy.dao.HpDocInfoMapper;
import cn.sh.stc.sict.theme.hphy.dao.HpHosInfoMapper;
import cn.sh.stc.sict.theme.hphy.model.HpDeptInfo;
import cn.sh.stc.sict.theme.hphy.model.HpDocInfo;
import cn.sh.stc.sict.theme.hphy.model.HpHosInfo;
import cn.sh.stc.sict.theme.hphy.service.HpDeptInfoService;
import cn.sh.stc.sict.theme.hphy.service.HpDocInfoService;
import cn.sh.stc.sict.theme.hphy.service.HpHosInfoService;
import cn.sh.stc.sict.theme.hphy.vo.DeptDoctorsVO;
import cn.sh.stc.sict.theme.hphy.wd.*;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * (HpDeptInfo)表服务实现类
 *
 * @author makejava
 * @since 2021-01-28 10:35:58
 */
@Slf4j
@AllArgsConstructor
@Service("hpDeptInfoService")
public class HpDeptInfoServiceImpl extends ServiceImpl<HpDeptInfoMapper, HpDeptInfo> implements HpDeptInfoService {
    private final HpDocInfoService hpDocInfoService;
    private final HpHosInfoService hpHosInfoService;

    @Override
    public DeptDoctorsVO getDeptDoctors(String hospitalCode, String deptCode) {
        HpHosInfo hosInfo = hpHosInfoService.getById(hospitalCode);
        // 查询科室信息
        // 查询医生信息
        HpDeptInfo deptInfo = this.getOne(
                new LambdaQueryWrapper<HpDeptInfo>()
                        .eq(HpDeptInfo::getHosOrgCode, hospitalCode)
                        .and(i -> i.eq(HpDeptInfo::getOneDeptCode, deptCode)
                                .or().eq(HpDeptInfo::getDeptCode, deptCode))
                        .orderByDesc(HpDeptInfo::getDeptLevel)
                        .last("limit 1")
        );
        if (deptInfo == null) {
            log.error("根据医院编码：{} 和科室编码：{} 未查出科室信息", hospitalCode, deptCode);
            return null;
        }

        LambdaQueryWrapper<HpDocInfo> docWrapper = new LambdaQueryWrapper<>();
        docWrapper.eq(HpDocInfo::getHosOrgCode, hospitalCode);
        docWrapper.eq(HpDocInfo::getOneDeptCode, deptInfo.getOneDeptCode());
        if (DataConstant.TWO_DEPT.equals(deptInfo.getDeptLevel())) {
            docWrapper.eq(HpDocInfo::getDeptCode, deptInfo.getDeptCode());
        }
        docWrapper.orderByAsc(HpDocInfo::getIndexNo);

        List<HpDocInfo> doctors = hpDocInfoService.list(docWrapper);


        DeptInfo info = new DeptInfo();
        info.setHosOrgCode(deptInfo.getHosOrgCode());
        info.setOneDeptCode(deptInfo.getOneDeptCode());
        info.setDeptCode(deptInfo.getDeptCode());
        List<OutPatInfo> resourceOutPatInfo = WanDaHttpUtil.getResourceOutPatInfo(info);
        List<DoctInfo> resourceDoctInfo = WanDaHttpUtil.getResourceDoctInfo(info);
        List<HpDocInfo> remainDoctors = this.getRemainDoctList(doctors, resourceDoctInfo);

        return new DeptDoctorsVO(hosInfo, deptInfo, resourceOutPatInfo, remainDoctors);
    }

    @Async
    @Override
    public void asyncUpdate(HpDeptInfo dept) {
        DeptInfo deptInfo = new DeptInfo();
        deptInfo.setHosOrgCode(dept.getHosOrgCode());
        deptInfo.setOneDeptCode(dept.getOneDeptCode());
        deptInfo.setDeptCode(dept.getDeptCode());
        List<OutPatInfo> resourceOutPatInfo = WanDaHttpUtil.getResourceOutPatInfo(deptInfo);
        List<DoctInfo> resourceDoctInfo = WanDaHttpUtil.getResourceDoctInfo(deptInfo);
        if (CollUtil.isEmpty(resourceDoctInfo) && CollUtil.isEmpty(resourceOutPatInfo)) {
            return;
        }
        HpDeptInfo update = new HpDeptInfo();
        update.setId(dept.getId());
        update.setResourceDoctInfo(JSONUtil.toJsonStr(resourceDoctInfo));
        update.setResourceOutPatInfo(JSONUtil.toJsonStr(resourceOutPatInfo));
        this.updateById(update);
    }

    private List<HpDocInfo> getRemainDoctList(List<HpDocInfo> doctors, List<DoctInfo> resourceDoctInfo) {
        Map<String, HpDocInfo> doctorMap = doctors.stream().collect(Collectors.groupingBy(HpDocInfo::getResourceCode, Collectors.collectingAndThen(Collectors.toList(), x -> x.get(0))));
        List<HpDocInfo> remainDoctors = new ArrayList<>();
        resourceDoctInfo.forEach(res -> {
            if(doctorMap.containsKey(res.getResourceCode())){
                remainDoctors.add(doctorMap.get(res.getResourceCode()));
            }else{
                HpDocInfo doctor = BeanUtil.toBean(res, HpDocInfo.class);
                hpDocInfoService.save(doctor);
                remainDoctors.add(doctor);
            }
        });

        return remainDoctors;
    }

//    public static void main(String[] args) {
//
//        DeptInfo info = new DeptInfo();
//        info.setHosOrgCode("Y0180100700");
//        info.setOneDeptCode("293");
//        info.setDeptCode("3");
//        List<OutPatInfo> resourceOutPatInfo = WanDaHttpUtil.getResourceOutPatInfo(info);
//        List<DoctInfo> resourceDoctInfo = WanDaHttpUtil.getResourceDoctInfo(info);
//        log.error("OutPatInfo = {}", JSONUtil.toJsonStr(resourceOutPatInfo));
//        log.error("=========================");
//        log.error("DoctInfo = {}", JSONUtil.toJsonStr(resourceDoctInfo));
//
//        DoctInfo doctInfo = resourceDoctInfo.get(0);
//        NumSourceInfo numSourceInfo = new NumSourceInfo();
//        numSourceInfo.setHosOrgCode(doctInfo.getHosOrgCode());
//        numSourceInfo.setOneDeptCode(doctInfo.getOneDeptCode());
//        numSourceInfo.setDeptCode(doctInfo.getDeptCode());
//        numSourceInfo.setOrderType(WanDaConstant.ORDER_TYPE_OUTP);
//        numSourceInfo.setResourceCode(doctInfo.getResourceCode());
//        numSourceInfo.setPatient_type(WanDaConstant.PATIENT_TYPE_NORMAL);
//        numSourceInfo.setStartTime("2022-07-25");
//        numSourceInfo.setEndTime("2022-08-08");
//
//        List<NumSourceInfo> orderNumInfo = WanDaHttpUtil.getOrderNumInfo(numSourceInfo);
//        log.error("=========================");
//        log.error("numInfo = {}", JSONUtil.toJsonStr(orderNumInfo));
//    }


//    public static void main(String[] args) {
//        NumSourceInfo numSourceInfo = new NumSourceInfo();
//        numSourceInfo.setHosOrgCode("E7880343800");
//        numSourceInfo.setOneDeptCode("50");
//        numSourceInfo.setDeptCode("2006");
//        numSourceInfo.setOrderType(WanDaConstant.ORDER_TYPE_OUTP);
//        numSourceInfo.setResourceCode("0085");
//        numSourceInfo.setPatient_type(WanDaConstant.PATIENT_TYPE_NORMAL);
//        Date startDate = DateUtil.date();
//        Date endDate = DateUtil.offsetDay(startDate, 10);
//        numSourceInfo.setStartTime(DateUtil.format(startDate, DatePattern.NORM_DATE_FORMAT));
//        numSourceInfo.setEndTime(DateUtil.format(endDate, DatePattern.NORM_DATE_FORMAT));
//
//        List<NumSourceInfo> list = WanDaHttpUtil.getOrderNumInfo(numSourceInfo);
//
//        log.error("list = {}", JSONUtil.toJsonStr(list));
//    }


    /**
     * 消化内科	42503527600	上海市瑞金医院卢湾分院	1013	3122	消化科	1	1013	3122
     * 消化内科	Y0180100700	上海交通大学医学院附属第九人民医院黄浦分院	293	3	消化内科	2	293	3
     * 消化内科	42502763900	黄浦区肿瘤防治院	10	0002	消化专科	3	10	0002
     * E7880343800 50	2006
     * @param args
     */
//    public static void main(String[] args) {
//        String orgCode = "E7880343800";
//        String oneDeptCode = "50";
//        String deptCode = "2006";
//
//        HosInfo hosInfo = new HosInfo();
//        hosInfo.setHosOrgCode(orgCode);
//        List<DeptInfo> oneDeptInfo = WanDaHttpUtil.getDeptInfoTop(hosInfo);
//        log.error("oneDeptInfo = {}", JSONUtil.toJsonStr(oneDeptInfo));
//        DeptInfo deptInfo = new DeptInfo();
//        deptInfo.setHosOrgCode(orgCode);
//        deptInfo.setOneDeptCode(oneDeptCode);
//        List<DeptInfo> deptInfoTwo = WanDaHttpUtil.getDeptInfoTwo(deptInfo);
//        log.error("deptInfoTwo = {}", JSONUtil.toJsonStr(deptInfoTwo));
//        deptInfo.setDeptCode(deptCode);
//        List<OutPatInfo> resourceOutPatInfo = WanDaHttpUtil.getResourceOutPatInfo(deptInfo);
//        log.error("resourceOutPatInfo = {}", JSONUtil.toJsonStr(resourceOutPatInfo));
//        List<DoctInfo> resourceDoctInfo = WanDaHttpUtil.getResourceDoctInfo(deptInfo);
//        log.error("resourceDoctInfo = {}", JSONUtil.toJsonStr(resourceDoctInfo));
//    }
}
